#$Header: /raid/cvsroot/DBIx/EasySearch/EasySearch.pm,v 1.8 2000/01/10 01:38:57 jesse Exp $

package DBIx::EasySearch;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;
require AutoLoader;

@ISA = qw(Exporter AutoLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(
	
);
$VERSION = '0.03';


# Preloaded methods go here.

#instantiate a new object.
sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self  = {};
  bless ($self, $class);
  $self->_Init(@_);
  return ($self)
}


#Initialize the object
sub _Init {
  my $self = shift;
  my %args = ( Handle => undef,
	       @_
	     );
  #print "ARGS Handle is $args{'Handle'} \n";
  $self->{'DBIxHandle'} = $args{'Handle'}; 
  $self->{'must_redo_search'}=1;
  $self->{'itemscount'}=0;
  $self->{'tables'} = "";
  $self->{'auxillary_tables'} = "";
  $self->{'where_clause'} = "";
  $self->{'table_links'} = "";
  $self->{'limit_clause'} = "";
  $self->{'order'} = "";
  #we have no limit statements. DoSearch won't work.
  $self->_isLimited(0);
}


sub _Handle {
  my $self = shift;
  return ($self->{'DBIxHandle'}->Handle);
}

#todo $self->item should point to our current fact.

sub Count {
  my $self = shift;
  
  if ($self->{'must_redo_search'}) {
    return ($self->_DoSearch);
  }
  else {
    return($self->{'rows'});
  }
}

sub _DoSearch {
  my $self = shift;
  my ($QueryString, $OrderClause);

  #compile all the standard restrictions
  $self->_CompileRestrictions();
  #compile the order by and order 
  if ($self->{'orderby'}) {
    $OrderClause = "ORDER BY ". $self->{'orderby'};
  }
  else {
    $OrderClause = "ORDER BY main.".$self->{'primary_key'}." ";
    
  }
  
  if ($self->{'order'} =~ /^des/) {
    $OrderClause .= " DESC";
  }
  else {
    $OrderClause .= " ASC";
    
  }


  
  $QueryString = "SELECT distinct main.".$self->{'primary_key'} ." FROM " .
    $self->{'table'}." AS main". $self->{'auxillary_tables'}." WHERE ".
      $self->{'where_clause'}." ".$self->{'table_links'}." ".
	$self->{'limit_clause'} . " ". $OrderClause;
  
  print STDERR "DBIx::EasySearch->DoSearch Query:  $QueryString\n";
  
  
  $self->{'records'} = $self->_Handle->prepare($QueryString);
  
  
  if (!$self->{'records'}) {
    die "Error:" . $self->_Handle->errstr . "\n";
  }
  if (!$self->{'records'}->execute) {
    die "DBIx::EasySearch error:" . $self->{'records'}->errstr . "\n\tQuery String is $QueryString\n";
  }
  
  
  $self->{'rows'} = $self->{'records'}->rows();
 # print STDERR "We got ",$self->{'rows'} ," rows\n";
 

  my $counter = 0;
  while (my @row = $self->{'records'}->fetchrow()) {
    $counter++;
    #print "Loading stuff # $counter in\n";
    #Call the childclass's newitem method
  
    $self->{'items'}[$counter] = $self->NewItem();
    
    
    #TODO THIS DOES NOT WORK.

    
    $self->{'items'}[$counter]->Load($row[0]);
    
#    print "ID is ". $self->{'items'}[$counter]->Id()."\n";
  }
  $self->{'must_redo_search'}=0;

  return($self->{'rows'});
  
  
}


sub Next {
  my $self = shift;
  
  my @row;

  if (!$self->_isLimited) {
    return(0);
  }
  if ($self->{'must_redo_search'} != 0) {
    $self->_DoSearch();
  }
  
  
  if ($self->{'itemscount'} < $self->{'rows'}) {
    
    #increment the itemcounter.
    $self->{'itemscount'}++;
    #serve out that item
    return ($self->{'items'}[$self->{'itemscount'}]);
  }
  else {
    #we've gone through the whole list.
    #reset the count.
    $self->{'itemscount'} = 0;
    return(undef);
    
  }
  
}




sub NewItem {
  my $self = shift;
  my $foo = (caller(1))[3];                                                        
  die "DBIx::EasySearch needs to be subclassed. you can't use it directly. ($foo)\n";
}

sub _isLimited {
  my $self = shift;
  if (@_) {
    $self->{'is_limited'} = shift;
  }
  else {
    return ($self->{'is_limited'});
  }
}

#####
#####Process LIMIT Statements 
#####This is where we do our magic
#####
sub Limit {
  my $self = shift;
  my %args = (
	      TABLE => $self->{'table'},
	      FIELD => undef,
	      VALUE => undef,
	      ALIAS => undef,
	      TYPE => undef, 
	      ENTRYAGGREGATOR => 'or',
	      INT_LINKFIELD => $self->{'primary_key'},
	      EXT_LINKFIELD => 'id',
	      OPERATOR => '=',
	      OFFSET => 0,
	      ORDERBY => undef,
	      ORDER => undef,
	      ROWS => undef,
	      @_ # get the real argumentlist
	     );
  
  my ($Alias);
  

  $self->_isLimited(1);
 
  #Do we want "ASC"ending order or "DESC"ending order
  if ($args{'ORDER'}) {
    $self->{'order'} = $args{'ORDER'};
  }
  #print STDERR "in DBIx::EasySearch->Limit: Field is $args{'FIELD'}\n";

  if ($args{'FIELD'}) {
    if ($args{'OPERATOR'} eq "LIKE") {
      $args{'VALUE'} = "%".$args{'VALUE'} ."%";
    }
    $args{'VALUE'} = $self->_Handle->quote($args{'VALUE'});
  }
  
  $Alias = $self->_GenericRestriction(%args);
  
  # Set the limit clause. used for restricting ourselves to subsets of the search.
  if ( $args{'ROWS'}) {
      $self->{'limit_clause'} = "LIMIT ";
      if ($args{'OFFSET'} != 0) {
	  $self->{'limit_clause'} .= $args{'OFFSET'} . ", ";
      }
    $self->{'limit_clause'} .= $args{'ROWS'};
  }
  
  #If we're setting an OrderBy, set it here
  # TODO: this may be broken. it needs to divine the right Column alias
  # it tries to be smart. here are the steps:
  # 1. if an alias was passed in, use that. 
  # 2. if an alias was generated, use that.
  # 3. use the primary
  if ($args{'ORDERBY'}) {
     if ($args{'ALIAS'}) {
       $self->{'orderby'} = $args{'ALIAS'}.".".$args{'ORDERBY'};
     }
     elsif ($Alias) {
       $self->{'orderby'} = "$Alias.".$args{'ORDERBY'};
     }
     else {
       $self->{'orderby'} = "main.".$args{'ORDERBY'};
     }
   }
  
  
  if ($Alias) {
      #ok. now we're limited. people can do searches.

    return($Alias);
  }
  else {
    return(0);
  }
}

  
#Show Restrictions
sub ShowRestrictions {
   my $self = shift;
  $self->_CompileRestrictions();
  return($self->{'where_clause'});
  
}


#import a restrictions clause

sub ImportRestrictions {
  my $self = shift;
  $self->{'where_clause'} = shift;
}



sub _GenericRestriction {
  my $self = shift;
  ;my %args = (
	      TABLE => $self->{'table'},
	      FIELD => undef,
	      VALUE => undef,   #TODO: $Value should take an array of values and generate the proper where clause.
	      ALIAS => undef,	     
	      ENTRYAGGREGATOR => undef,
	      OPERATOR => '=',
	       INT_LINKFIELD => undef,
	       EXT_LINKFIELD => undef,
	      @_);
    my ($QualifiedField);
  
  #since we're changing the search criteria, we need to redo the search
  $self->{'must_redo_search'}=1;
  

#if the operator is a like, we need to add %s on ether side of the VALUE

  # if there's no alias set, we need to set it
  if (!$args{'ALIAS'}) {
    
    #if the table we're looking at is the same as the main table
    if ($args{'TABLE'} eq $self->{'table'}) {
      
      # main is the alias of the "primary table. this code assumes no self 
      # joins on that table. if someone can name   case where we'd want to do that, I'll change it.
      
      $args{'ALIAS'} = 'main';
    }
    
    # if we're joining, we need to work out the table alias
    else {
      $self->{'aliascount'}++;
      $args{'ALIAS'} = $args{'TABLE'}."_".$self->{'aliascount'};
      $self->{'auxillary_tables'} .= ",".$args{'TABLE'}." as $args{'ALIAS'}";
      
      #      if ($self->{'table_links'} ne undef) {
      
      # do this every time. we need the initial and as well as any joining ands
      $self->{'table_links'} .= ' AND ';
      
      #      }
      #     else {
      #supress a warning
      #	$self->{'table_links'} = "";
      #      }
      
      # we need to build the table of links.
      $self->{'table_links'} .= " main.". $args{'INT_LINKFIELD'}."=".$args{'ALIAS'}.".".$args{'EXT_LINKFIELD'};
      
      
    }
  }
  #If we were just setting an alias, return
  #TODO: all code above this point should be seperate
  if (!$args{'FIELD'}) {
    return ($args{'ALIAS'});
    
  }
  
  #Set this to the name of the field and the alias.
  $QualifiedField = $args{'ALIAS'}.".".$args{'FIELD'};
 # print STDERR "DBIx::EasySearch->_GenericRestriction  QualifiedField is $QualifiedField\n";:
  
  #If we're overwriting this sort of restriction, 
  if (($args{'ENTRYAGGREGATOR'} eq 'none') or 
      (!$self->{'restrictions'}{"$QualifiedField"})) {
    $self->{'restrictions'}{"$QualifiedField"} = 
      "($QualifiedField $args{'OPERATOR'} $args{'VALUE'})";  
  #  print STDERR "self->{'restrictions'}{$QualifiedField} cmpval is ($QualifiedField $args{'OPERATOR'} $args{'VALUE'})";  

  }
  else {
    $self->{'restrictions'}{"$QualifiedField"} .= 
      " $args{'ENTRYAGGREGATOR'} ($QualifiedField $args{'OPERATOR'} $args{'VALUE'})";
   # print STDERR "self->{'restrictions'}{$QualifiedField} cmpval is added to by: ($QualifiedField $args{'OPERATOR'} $args{'VALUE'})"; 
  }

 return ($args{'ALIAS'});
 
}
  
#Compile the restrictions to a WHERE Clause


sub _CompileRestrictions {
  my $self = shift;
  my ($restriction, $RelType);
  
  $self->{'where_clause'} = "";
  
  foreach $restriction (keys %{ $self->{'restrictions'}}) {
    
    if ($self->{'where_clause'} ne '') {
      $self->{'where_clause'} .= " AND ";
      }
    $self->{'where_clause'} .= "" . $self->{'restrictions'}{"$restriction"} . "";
  }
  
}




# Autoload methods go after =cut, and are processed by the autosplit program.


1;
__END__

=head1 NAME

DBIx::EasySearch - Perl extension for easy SQL SELECT Statement generation

=head1 SYNOPSIS

  use DBIx::EasySearch;

  then read the code. (yes, i'm being lame)

=head1 DESCRIPTION

Jesse is lame and hasn't written docs yet

=head1 AUTHOR

Jesse Vincent, jesse@fsck.com

=head1 SEE ALSO

DBIx::Handle, DBIx::Record, perl(1).

=cut
